@@ -8,6 +8,8 @@ use std::{
 };
 use std::sync::{Arc, Mutex};
 
+use async_trait::async_trait;
+use log::{debug, warn};
 use url::Url;
 
 use pact_mock_server::*;
@@ -15,16 +17,28 @@ use pact_mock_server::matching::MatchResult;
 use pact_mock_server::mock_server::{MockServerConfig, MockServerMetrics};
 use pact_models::pact::Pact;
 use pact_models::sync_pact::RequestResponsePact;
+use std::path::PathBuf;
+use uuid::Uuid;
+use pact_matching::metrics::{MetricEvent, send_metrics};
 
 /// This trait is implemented by types which allow us to start a mock server.
+#[async_trait]
 pub trait StartMockServer {
-    /// Start a mock server running in a background thread.
-    fn start_mock_server(&self) -> ValidatingMockServer;
+  /// Start a mock server running in a background thread.
+  fn start_mock_server(&self) -> ValidatingMockServer;
+
+  /// Start a mock server running in a task (requires a Tokio runtime to be already setup)
+  async fn start_mock_server_async(&self) -> ValidatingMockServer;
 }
 
+#[async_trait]
 impl StartMockServer for RequestResponsePact {
   fn start_mock_server(&self) -> ValidatingMockServer {
-    ValidatingMockServer::start(self.boxed())
+    ValidatingMockServer::start(self.boxed(), None)
+  }
+
+  async fn start_mock_server_async(&self) -> ValidatingMockServer {
+    ValidatingMockServer::start_async(self.boxed(), None).await
   }
 }
 
@@ -43,14 +57,20 @@ pub struct ValidatingMockServer {
     mock_server: Arc<Mutex<mock_server::MockServer>>,
     // Signal received when the server thread is done executing
     done_rx: std::sync::mpsc::Receiver<()>,
+    // Output directory to write pact files
+    output_dir: Option<PathBuf>,
 }
 
 impl ValidatingMockServer {
   /// Create a new mock server which handles requests as described in the
   /// pact, and runs in a background thread
-  pub fn start(pact: Box<dyn Pact + Send>) -> ValidatingMockServer {
+  ///
+  /// Panics:
+  /// Will panic if the provided Pact can not be sent to the background thread.
+  pub fn start(pact: Box<dyn Pact + Send + Sync>, output_dir: Option<PathBuf>) -> ValidatingMockServer {
+    debug!("Starting mock server from pact {:?}", pact);
     // Spawn new runtime in thread to prevent reactor execution context conflict
-    let (pact_tx, pact_rx) = std::sync::mpsc::channel::<Box<dyn Pact + Send>>();
+    let (pact_tx, pact_rx) = std::sync::mpsc::channel::<Box<dyn Pact + Send + Sync>>();
     pact_tx.send(pact).expect("INTERNAL ERROR: Could not pass pact into mock server thread");
     let (mock_server, done_rx) = std::thread::spawn(|| {
       let runtime = tokio::runtime::Builder::new_current_thread()
@@ -59,8 +79,12 @@ impl ValidatingMockServer {
         .expect("new runtime");
 
       let (mock_server, server_future) = runtime.block_on(async move {
-        mock_server::MockServer::new("".into(), pact_rx.recv().unwrap(), ([0, 0, 0, 0], 0).into(),
-          MockServerConfig::default())
+        mock_server::MockServer::new(
+          Uuid::new_v4().to_string(),
+          pact_rx.recv().unwrap(),
+          ([0, 0, 0, 0], 0).into(),
+          MockServerConfig::default()
+        )
           .await
           .unwrap()
       });
@@ -97,6 +121,47 @@ impl ValidatingMockServer {
       url: url_str.parse().expect("invalid mock server URL"),
       mock_server,
       done_rx,
+      output_dir
+    }
+  }
+
+  /// Create a new mock server which handles requests as described in the
+  /// pact, and runs in a background task in the current Tokio runtime.
+  ///
+  /// Panics:
+  /// Will panic if unable to get the URL to the spawned mock server
+  pub async fn start_async(pact: Box<dyn Pact + Send + Sync>, output_dir: Option<PathBuf>) -> ValidatingMockServer {
+    debug!("Starting mock server from pact {:?}", pact);
+
+    let (mock_server, server_future) = mock_server::MockServer::new(
+      Uuid::new_v4().to_string(),
+      pact,
+      ([0, 0, 0, 0], 0 as u16).into(),
+      MockServerConfig::default()
+    )
+      .await
+      .unwrap();
+
+    let (done_tx, done_rx) = std::sync::mpsc::channel::<()>();
+    tokio::spawn(async move {
+      dbg!(server_future.await);
+      let _ = done_tx.send(());
+    });
+
+    let (description, url_str) = {
+      let ms = mock_server.lock().unwrap();
+      let pact = ms.pact.lock().unwrap();
+      let description = format!(
+        "{}/{}", pact.consumer().name, pact.provider().name
+      );
+      (description, ms.url())
+    };
+    ValidatingMockServer {
+      description,
+      url: url_str.parse().expect("invalid mock server URL"),
+      mock_server,
+      done_rx,
+      output_dir
     }
   }
 
@@ -135,19 +200,37 @@ impl ValidatingMockServer {
         }
 
         // Wait for the server thread to finish
-        self.done_rx
-            .recv_timeout(std::time::Duration::from_secs(3))
-            .expect("mock server thread should not panic");
+        if let Err(_) = self.done_rx.recv_timeout(std::time::Duration::from_secs(3)) {
+          warn!("Timed out waiting for mock server to finish");
+        }
+
+      let interactions = {
+        let pact = ms.pact.lock().unwrap();
+        pact.interactions().len()
+      };
+      send_metrics(MetricEvent::ConsumerTestRun {
+        interactions,
+        test_framework: "pact_consumer".to_string(),
+        app_name: "pact_consumer".to_string(),
+        app_version: env!("CARGO_PKG_VERSION").to_string()
+      });
 
         // Look up any mismatches which occurred.
         let mismatches = ms.mismatches();
 
         if mismatches.is_empty() {
-            // Success! Write out the generated pact file.
-            ms.write_pact(
-              &Some(
-                env::var("PACT_OUTPUT_DIR").unwrap_or_else(|_| "target/pacts".to_owned())),
-              env::var("PACT_OVERWRITE").unwrap_or_else(|_| "false".to_owned()) == "true")
+          // Success! Write out the generated pact file.
+          let output_dir = self.output_dir.as_ref().map(|dir| dir.to_string_lossy().to_string())
+            .unwrap_or_else(|| {
+              let val = env::var("PACT_OUTPUT_DIR");
+              debug!("env:PACT_OUTPUT_DIR = {:?}", val);
+              val.unwrap_or_else(|_| "target/pacts".to_owned())
+            });
+          let overwrite = env::var("PACT_OVERWRITE");
+          debug!("env:PACT_OVERWRITE = {:?}", overwrite);
+          ms.write_pact(
+            &Some(output_dir),
+            overwrite.unwrap_or_else(|_| "false".to_owned()) == "true")
             .map_err(|err| format!("error writing pact: {}", err))?;
             Ok(())
         } else {
